import CthulhuEternalActorSheet from "./base-actor-sheet.mjs"

export default class CthulhuEternalProtagonistSheet extends CthulhuEternalActorSheet {
  /** @override */
  static DEFAULT_OPTIONS = {
    classes: ["protagonist"],
    position: {
      width: 1150,
      height: 780,
    },
    window: {
      contentClasses: ["protagonist-content"],
    },
    actions: {
      createEquipment: CthulhuEternalProtagonistSheet.#onCreateEquipment,
    },
  }

  /** @override */
  static PARTS = {
    main: {
      template: "systems/fvtt-cthulhu-eternal/templates/protagonist-main.hbs",
    },
    tabs: {
      template: "templates/generic/tab-navigation.hbs",
    },
    skills: {
      template: "systems/fvtt-cthulhu-eternal/templates/protagonist-skills.hbs",
    },
    equipment: {
      template: "systems/fvtt-cthulhu-eternal/templates/protagonist-equipment.hbs",
    },
    biography: {
      template: "systems/fvtt-cthulhu-eternal/templates/protagonist-biography.hbs",
    },
  }

  /** @override */
  tabGroups = {
    sheet: "skills",
  }

  /**
   * Prepare an array of form header tabs.
   * @returns {Record<string, Partial<ApplicationTab>>}
   */
  #getTabs() {
    const tabs = {
      skills: { id: "skills", group: "sheet", icon: "fa-solid fa-shapes", label: "CTHULHUETERNAL.Label.skills" },
      equipment: { id: "equipment", group: "sheet", icon: "fa-solid fa-shapes", label: "CTHULHUETERNAL.Label.equipment" },
      biography: { id: "biography", group: "sheet", icon: "fa-solid fa-book", label: "CTHULHUETERNAL.Label.biography" },
    }
    for (const v of Object.values(tabs)) {
      v.active = this.tabGroups[v.group] === v.id
      v.cssClass = v.active ? "active" : ""
    }
    return tabs
  }

  /** @override */
  async _prepareContext() {
    const context = await super._prepareContext()
    context.tabs = this.#getTabs()

    context.tooltipsCharacteristic = {
      str: game.i18n.localize("CTHULHUETERNAL.Characteristic.Str"),
      dex: game.i18n.localize("CTHULHUETERNAL.Characteristic.Dex"),
      con: game.i18n.localize("CTHULHUETERNAL.Characteristic.Con"),
      int: game.i18n.localize("CTHULHUETERNAL.Characteristic.Int"),
      pow: game.i18n.localize("CTHULHUETERNAL.Characteristic.Pow"),
      cha: game.i18n.localize("CTHULHUETERNAL.Characteristic.Cha")
    }

    context.tooltipsRessources = {
    }

    context.rollType = {
      str: "characteristic",
      dex: "characteristic",
      con: "characteristic",
      int: "characteristic",
      pow: "characteristic",
      cha: "characteristic"
    }
    return context
  }

  _generateTooltip(type, target) {
  }

  /** @override */
  async _preparePartContext(partId, context) {
    const doc = this.document
    switch (partId) {
      case "main":
        break
      case "skills":
        context.tab = context.tabs.skills
        context.skills = doc.itemTypes.skill
        break
      case "equipment":
        context.tab = context.tabs.equipment
        context.weapons = doc.itemTypes.weapon
        context.armors = doc.itemTypes.armor
        context.gears = doc.itemTypes.gear
        break
      case "biography":
        context.tab = context.tabs.biography
        context.enrichedDescription = await TextEditor.enrichHTML(doc.system.description, { async: true })
        context.enrichedNotes = await TextEditor.enrichHTML(doc.system.notes, { async: true })
        break
    }
    return context
  }

  /**
   * Creates a new attack item directly from the sheet and embeds it into the document.
   * @param {Event} event             The initiating click event.
   * @param {HTMLElement} target      The current target of the event listener.
   */
  static #onCreateEquipment(event, target) {
    // Création d'une armure
    if (event.shiftKey) {
      this.document.createEmbeddedDocuments("Item", [{ name: game.i18n.localize("CTHULHUETERNAL.Label.newArmor"), type: "armor" }])
    }
    // Création d'une arme
    else {
      this.document.createEmbeddedDocuments("Item", [{ name: game.i18n.localize("CTHULHUETERNAL.Label.newWeapon"), type: "weapon" }])
    }
  }

  /**
   * Handles the roll action triggered by user interaction.
   *
   * @param {PointerEvent} event The event object representing the user interaction.
   * @param {HTMLElement} target The target element that triggered the roll.
   *
   * @returns {Promise<void>} A promise that resolves when the roll action is complete.
   *
   * @throws {Error} Throws an error if the roll type is not recognized.
   *
   * @description This method checks the current mode (edit or not) and determines the type of roll
   * (save, resource, or damage) based on the target element's data attributes. It retrieves the
   * corresponding value from the document's system and performs the roll.
   */
  async _onRoll(event, target) {
    if (this.isEditMode) return
    // Jet de sauvegarde
    let elt = event.currentTarget.querySelector("input")
    // Jet de ressource
    if (!elt) elt = event.currentTarget.querySelector("select")
    // Jet de dégâts
    if (!elt) elt = event.currentTarget
    const rollType = elt.dataset.rollType
    let rollTarget
    switch (rollType) {
      default:
        break
    }
    await this.document.system.roll(rollType, rollTarget)
  }

  async _onDrop(event) {
    if (!this.isEditable || !this.isEditMode) return
    const data = TextEditor.getDragEventData(event)

    // Handle different data types
    switch (data.type) {
      case "Item":
        const item = await fromUuid(data.uuid)
        return super._onDropItem(item)
    }
  }

  // #endregion
}
